@@ -21,18 +21,15 @@ package org.apache.pulsar;
 import static org.apache.commons.lang3.StringUtils.isBlank;
 import com.beust.jcommander.JCommander;
 import com.beust.jcommander.Parameter;
-import com.google.common.annotations.VisibleForTesting;
 import com.google.common.base.Strings;
 import java.io.FileInputStream;
 import java.util.Arrays;
-import lombok.AccessLevel;
-import lombok.Setter;
 import lombok.extern.slf4j.Slf4j;
 import org.apache.commons.lang3.StringUtils;
 import org.apache.logging.log4j.LogManager;
 import org.apache.pulsar.broker.ServiceConfiguration;
 import org.apache.pulsar.common.configuration.PulsarConfigurationLoader;
-import org.apache.pulsar.common.util.CmdGenerateDocs;
+import org.apache.pulsar.docs.tools.CmdGenerateDocs;
 
 @Slf4j
 public class PulsarStandaloneStarter extends PulsarStandalone {
@@ -41,9 +38,6 @@ public class PulsarStandaloneStarter extends PulsarStandalone {
 
     @Parameter(names = {"-g", "--generate-docs"}, description = "Generate docs")
     private boolean generateDocs = false;
-    private Thread shutdownThread;
-    @Setter(AccessLevel.PACKAGE)
-    private boolean testMode;
 
     public PulsarStandaloneStarter(String[] args) throws Exception {
 
@@ -114,50 +108,26 @@ public class PulsarStandaloneStarter extends PulsarStandalone {
                 }
             }
         }
-    }
 
-    @Override
-    public synchronized void start() throws Exception {
-        super.start();
-        if (shutdownThread != null) {
-            throw new IllegalStateException("Shutdown hook already registered");
-        }
-        shutdownThread = new Thread(() -> {
+        Runtime.getRuntime().addShutdownHook(new Thread(() -> {
             try {
-                doClose(false);
+                if (fnWorkerService != null) {
+                    fnWorkerService.stop();
+                }
+
+                if (broker != null) {
+                    broker.close();
+                }
+
+                if (bkEnsemble != null) {
+                    bkEnsemble.stop();
+                }
             } catch (Exception e) {
                 log.error("Shutdown failed: {}", e.getMessage(), e);
             } finally {
-                if (!testMode) {
-                    LogManager.shutdown();
-                }
+                LogManager.shutdown();
             }
-        });
-        Runtime.getRuntime().addShutdownHook(shutdownThread);
-    }
-
-    // simulate running the shutdown hook, for testing
-    @VisibleForTesting
-    void runShutdownHook() {
-        if (!testMode) {
-            throw new IllegalStateException("Not in test mode");
-        }
-        Runtime.getRuntime().removeShutdownHook(shutdownThread);
-        shutdownThread.run();
-        shutdownThread = null;
-    }
-
-    @Override
-    public void close() {
-        doClose(true);
-    }
-
-    private synchronized void doClose(boolean removeShutdownHook) {
-        super.close();
-        if (shutdownThread != null && removeShutdownHook) {
-            Runtime.getRuntime().removeShutdownHook(shutdownThread);
-            shutdownThread = null;
-        }
+        }));
     }
 
     private static boolean argsContains(String[] args, String arg) {
