@@ -19,7 +19,6 @@
 package org.apache.pulsar.broker.service.persistent;
 
 import static org.apache.pulsar.broker.service.persistent.PersistentTopic.MESSAGE_RATE_BACKOFF_MS;
-import com.google.common.annotations.VisibleForTesting;
 import com.google.common.collect.Lists;
 import com.google.common.collect.Range;
 import java.util.ArrayList;
@@ -38,7 +37,6 @@ import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.concurrent.atomic.AtomicInteger;
 import java.util.concurrent.atomic.AtomicIntegerFieldUpdater;
 import java.util.function.Predicate;
-import lombok.Getter;
 import org.apache.bookkeeper.mledger.AsyncCallbacks.ReadEntriesCallback;
 import org.apache.bookkeeper.mledger.Entry;
 import org.apache.bookkeeper.mledger.ManagedCursor;
@@ -70,11 +68,11 @@ import org.apache.pulsar.broker.service.StickyKeyConsumerSelector;
 import org.apache.pulsar.broker.service.Subscription;
 import org.apache.pulsar.broker.service.persistent.DispatchRateLimiter.Type;
 import org.apache.pulsar.broker.transaction.exception.buffer.TransactionBufferException;
+import org.apache.pulsar.client.impl.Backoff;
 import org.apache.pulsar.common.api.proto.CommandSubscribe.SubType;
 import org.apache.pulsar.common.api.proto.MessageMetadata;
 import org.apache.pulsar.common.policies.data.stats.TopicMetricBean;
 import org.apache.pulsar.common.protocol.Commands;
-import org.apache.pulsar.common.util.Backoff;
 import org.apache.pulsar.common.util.Codec;
 import org.apache.pulsar.common.util.FutureUtil;
 import org.slf4j.Logger;
@@ -100,7 +98,6 @@ public class PersistentDispatcherMultipleConsumers extends AbstractDispatcherMul
     protected volatile boolean havePendingReplayRead = false;
     protected volatile PositionImpl minReplayedPosition = null;
     protected boolean shouldRewindBeforeReadingOrReplaying = false;
-    @Getter
     protected final String name;
     private boolean sendInProgress = false;
     protected static final AtomicIntegerFieldUpdater<PersistentDispatcherMultipleConsumers>
@@ -148,7 +145,7 @@ public class PersistentDispatcherMultipleConsumers extends AbstractDispatcherMul
                 : RedeliveryTrackerDisabled.REDELIVERY_TRACKER_DISABLED;
         this.readBatchSize = serviceConfig.getDispatcherMaxReadBatchSize();
         this.initializeDispatchRateLimiterIfNeeded();
-        this.assignor = new SharedConsumerAssignor(this::getNextConsumer, this::addMessageToReplay, subscription);
+        this.assignor = new SharedConsumerAssignor(this::getNextConsumer, this::addMessageToReplay);
         this.readFailureBackoff = new Backoff(
                 topic.getBrokerService().pulsar().getConfiguration().getDispatcherReadFailureBackoffInitialTimeInMs(),
                 TimeUnit.MILLISECONDS,
@@ -180,15 +177,9 @@ public class PersistentDispatcherMultipleConsumers extends AbstractDispatcherMul
         }
 
         if (isConsumersExceededOnSubscription()) {
-            log.warn("[{}] Attempting to add consumer to subscription which reached max consumers limit {}",
-                    name, consumer);
+            log.warn("[{}] Attempting to add consumer to subscription which reached max consumers limit", name);
             return FutureUtil.failedFuture(new ConsumerBusyException("Subscription reached max consumers limit"));
         }
-        // This is not an expected scenario, it will never happen in expected. Just print a warn log if the unexpected
-        // scenario happens. See more detail: https://github.com/apache/pulsar/pull/22283.
-        if (consumerSet.contains(consumer)) {
-            log.warn("[{}] Attempting to add a consumer that already registered {}", name, consumer);
-        }
 
         consumerList.add(consumer);
         if (consumerList.size() > 1
@@ -213,7 +204,15 @@ public class PersistentDispatcherMultipleConsumers extends AbstractDispatcherMul
             consumerList.remove(consumer);
             log.info("Removed consumer {} with pending {} acks", consumer, consumer.getPendingAcks().size());
             if (consumerList.isEmpty()) {
-                clearComponentsAfterRemovedAllConsumers();
+                cancelPendingRead();
+
+                redeliveryMessages.clear();
+                redeliveryTracker.clear();
+                if (closeFuture != null) {
+                    log.info("[{}] All consumers removed. Subscription is disconnected", name);
+                    closeFuture.complete(null);
+                }
+                totalAvailablePermits = 0;
             } else {
                 if (log.isDebugEnabled()) {
                     log.debug("[{}] Consumer are left, reading more entries", name);
@@ -230,31 +229,10 @@ public class PersistentDispatcherMultipleConsumers extends AbstractDispatcherMul
                 readMoreEntries();
             }
         } else {
-            /**
-             * This is not an expected scenario, it will never happen in expected.
-             * Just add a defensive code to avoid the topic can not be unloaded anymore: remove the consumers which
-             * are not mismatch with {@link #consumerSet}. See more detail: https://github.com/apache/pulsar/pull/22270.
-             */
-            log.error("[{}] Trying to remove a non-connected consumer: {}", name, consumer);
-            consumerList.removeIf(c -> consumer.equals(c));
-            if (consumerList.isEmpty()) {
-                clearComponentsAfterRemovedAllConsumers();
-            }
+            log.info("[{}] Trying to remove a non-connected consumer: {}", name, consumer);
         }
     }
 
-    private synchronized void clearComponentsAfterRemovedAllConsumers() {
-        cancelPendingRead();
-
-        redeliveryMessages.clear();
-        redeliveryTracker.clear();
-        if (closeFuture != null) {
-            log.info("[{}] All consumers removed. Subscription is disconnected", name);
-            closeFuture.complete(null);
-        }
-        totalAvailablePermits = 0;
-    }
-
     @Override
     public void consumerFlow(Consumer consumer, int additionalNumberOfMessages) {
         topic.getBrokerService().executor().execute(() -> {
@@ -289,12 +267,6 @@ public class PersistentDispatcherMultipleConsumers extends AbstractDispatcherMul
     }
 
     public synchronized void readMoreEntries() {
-        if (cursor.isClosed()) {
-            if (log.isDebugEnabled()) {
-                log.debug("[{}] Cursor is already closed, skipping read more entries.", cursor.getName());
-            }
-            return;
-        }
         if (isSendInProgress()) {
             // we cannot read more entries while sending the previous batch
             // otherwise we could re-read the same entries and send duplicates
@@ -318,25 +290,24 @@ public class PersistentDispatcherMultipleConsumers extends AbstractDispatcherMul
             }
 
             NavigableSet<PositionImpl> messagesToReplayNow = getMessagesToReplayNow(messagesToRead);
-            NavigableSet<PositionImpl> messagesToReplayFiltered = filterOutEntriesWillBeDiscarded(messagesToReplayNow);
-            if (!messagesToReplayFiltered.isEmpty()) {
+
+            if (!messagesToReplayNow.isEmpty()) {
                 if (log.isDebugEnabled()) {
-                    log.debug("[{}] Schedule replay of {} messages for {} consumers", name,
-                            messagesToReplayFiltered.size(), consumerList.size());
+                    log.debug("[{}] Schedule replay of {} messages for {} consumers", name, messagesToReplayNow.size(),
+                            consumerList.size());
                 }
 
                 havePendingReplayRead = true;
                 minReplayedPosition = messagesToReplayNow.first();
                 Set<? extends Position> deletedMessages = topic.isDelayedDeliveryEnabled()
-                        ? asyncReplayEntriesInOrder(messagesToReplayFiltered)
-                        : asyncReplayEntries(messagesToReplayFiltered);
+                        ? asyncReplayEntriesInOrder(messagesToReplayNow) : asyncReplayEntries(messagesToReplayNow);
                 // clear already acked positions from replay bucket
 
                 deletedMessages.forEach(position -> redeliveryMessages.remove(((PositionImpl) position).getLedgerId(),
                         ((PositionImpl) position).getEntryId()));
                 // if all the entries are acked-entries and cleared up from redeliveryMessages, try to read
                 // next entries as readCompletedEntries-callback was never called
-                if ((messagesToReplayFiltered.size() - deletedMessages.size()) == 0) {
+                if ((messagesToReplayNow.size() - deletedMessages.size()) == 0) {
                     havePendingReplayRead = false;
                     readMoreEntriesAsync();
                 }
@@ -345,7 +316,7 @@ public class PersistentDispatcherMultipleConsumers extends AbstractDispatcherMul
                     log.debug("[{}] Dispatcher read is blocked due to unackMessages {} reached to max {}", name,
                             totalUnackedMessages, topic.getMaxUnackedMessagesOnSubscription());
                 }
-            } else if (!havePendingRead && hasConsumersNeededNormalRead()) {
+            } else if (!havePendingRead) {
                 if (log.isDebugEnabled()) {
                     log.debug("[{}] Schedule read of {} messages for {} consumers", name, messagesToRead,
                             consumerList.size());
@@ -374,16 +345,7 @@ public class PersistentDispatcherMultipleConsumers extends AbstractDispatcherMul
                             topic.getMaxReadPosition());
                 }
             } else {
-                if (log.isDebugEnabled()) {
-                    if (!messagesToReplayNow.isEmpty()) {
-                        log.debug("[{}] [{}] Skipping read for the topic: because all entries in replay queue were"
-                                + " filtered out due to the mechanism of Key_Shared mode, and the left consumers have"
-                                + " no permits now",
-                                topic.getName(), getSubscriptionName());
-                    } else {
-                        log.debug("[{}] Cannot schedule next read until previous one is done", name);
-                    }
-                }
+                log.debug("[{}] Cannot schedule next read until previous one is done", name);
             }
         } else {
             if (log.isDebugEnabled()) {
@@ -544,9 +506,6 @@ public class PersistentDispatcherMultipleConsumers extends AbstractDispatcherMul
         if (consumerList.isEmpty()) {
             closeFuture.complete(null);
         } else {
-            // Iterator of CopyOnWriteArrayList uses the internal array to do the for-each, and CopyOnWriteArrayList
-            // will create a new internal array when adding/removing a new item. So remove items in the for-each
-            // block is safety when the for-each and add/remove are using a same lock.
             consumerList.forEach(consumer -> consumer.disconnect(isResetCursor));
             cancelPendingRead();
         }
@@ -862,14 +821,7 @@ public class PersistentDispatcherMultipleConsumers extends AbstractDispatcherMul
         ReadType readType = (ReadType) ctx;
         long waitTimeMillis = readFailureBackoff.next();
 
-        // Do not keep reading more entries if the cursor is already closed.
-        if (exception instanceof ManagedLedgerException.CursorAlreadyClosedException) {
-            if (log.isDebugEnabled()) {
-                log.debug("[{}] Cursor is already closed, skipping read more entries", cursor.getName());
-            }
-            // Set the wait time to -1 to avoid rescheduling the read.
-            waitTimeMillis = -1;
-        } else if (exception instanceof NoMoreEntriesToReadException) {
+        if (exception instanceof NoMoreEntriesToReadException) {
             if (cursor.getNumberOfEntriesInBacklog(false) == 0) {
                 // Topic has been terminated and there are no more entries to read
                 // Notify the consumer only if all the messages were already acknowledged
@@ -908,14 +860,7 @@ public class PersistentDispatcherMultipleConsumers extends AbstractDispatcherMul
         }
 
         readBatchSize = serviceConfig.getDispatcherMinReadBatchSize();
-        // Skip read if the waitTimeMillis is a nagetive value.
-        if (waitTimeMillis >= 0) {
-            scheduleReadEntriesWithDelay(exception, readType, waitTimeMillis);
-        }
-    }
 
-    @VisibleForTesting
-    void scheduleReadEntriesWithDelay(Exception e, ReadType readType, long waitTimeMillis) {
         topic.getBrokerService().executor().schedule(() -> {
             synchronized (PersistentDispatcherMultipleConsumers.this) {
                 // If it's a replay read we need to retry even if there's already
@@ -925,10 +870,11 @@ public class PersistentDispatcherMultipleConsumers extends AbstractDispatcherMul
                     log.info("[{}] Retrying read operation", name);
                     readMoreEntries();
                 } else {
-                    log.info("[{}] Skipping read retry: havePendingRead {}", name, havePendingRead, e);
+                    log.info("[{}] Skipping read retry: havePendingRead {}", name, havePendingRead, exception);
                 }
             }
         }, waitTimeMillis, TimeUnit.MILLISECONDS);
+
     }
 
     private boolean needTrimAckedMessages() {
@@ -1105,15 +1051,15 @@ public class PersistentDispatcherMultipleConsumers extends AbstractDispatcherMul
         }
 
         synchronized (this) {
-            if (delayedDeliveryTracker.isEmpty()) {
+            if (!delayedDeliveryTracker.isPresent()) {
                 if (!msgMetadata.hasDeliverAtTime()) {
                     // No need to initialize the tracker here
                     return false;
                 }
 
                 // Initialize the tracker the first time we need to use it
-                delayedDeliveryTracker = Optional.of(
-                        topic.getBrokerService().getDelayedDeliveryTrackerFactory().newTracker(this));
+                delayedDeliveryTracker = Optional
+                        .of(topic.getBrokerService().getDelayedDeliveryTrackerFactory().newTracker(this));
             }
 
             delayedDeliveryTracker.get().resetTickTime(topic.getDelayedDeliveryTickTimeMillis());
@@ -1138,27 +1084,6 @@ public class PersistentDispatcherMultipleConsumers extends AbstractDispatcherMul
         }
     }
 
-    /**
-     * This is a mode method designed for Key_Shared mode.
-     * Filter out the entries that will be discarded due to the order guarantee mechanism of Key_Shared mode.
-     * This method is in order to avoid the scenario below:
-     * - Get positions from the Replay queue.
-     * - Read entries from BK.
-     * - The order guarantee mechanism of Key_Shared mode filtered out all the entries.
-     * - Delivery non entry to the client, but we did a BK read.
-     */
-    protected NavigableSet<PositionImpl> filterOutEntriesWillBeDiscarded(NavigableSet<PositionImpl> src) {
-        return src;
-    }
-
-    /**
-     * This is a mode method designed for Key_Shared mode, to avoid unnecessary stuck.
-     * See detail {@link PersistentStickyKeyDispatcherMultipleConsumers#hasConsumersNeededNormalRead}.
-     */
-    protected boolean hasConsumersNeededNormalRead() {
-        return true;
-    }
-
     protected synchronized boolean shouldPauseDeliveryForDelayTracker() {
         return delayedDeliveryTracker.isPresent() && delayedDeliveryTracker.get().shouldPauseAllDeliveries();
     }
@@ -1229,7 +1154,7 @@ public class PersistentDispatcherMultipleConsumers extends AbstractDispatcherMul
             return false;
         }
         // consider dispatch is stuck if : dispatcher has backlog, available-permits and there is no pending read
-        if (isAtleastOneConsumerAvailable() && !havePendingReplayRead && !havePendingRead
+        if (totalAvailablePermits > 0 && !havePendingReplayRead && !havePendingRead
                 && cursor.getNumberOfEntriesInBacklog(false) > 0) {
             log.warn("{}-{} Dispatcher is stuck and unblocking by issuing reads", topic.getName(), name);
             readMoreEntries();
@@ -1267,14 +1192,5 @@ public class PersistentDispatcherMultipleConsumers extends AbstractDispatcherMul
         return StickyKeyConsumerSelector.makeStickyKeyHash(peekStickyKey(entry.getDataBuffer()));
     }
 
-
-    public Subscription getSubscription() {
-        return subscription;
-    }
-
-    public long getNumberOfMessagesInReplay() {
-        return redeliveryMessages.size();
-    }
-
     private static final Logger log = LoggerFactory.getLogger(PersistentDispatcherMultipleConsumers.class);
 }
