@@ -20,38 +20,23 @@ package org.apache.pulsar.broker.web;
 
 import io.prometheus.client.CollectorRegistry;
 import io.prometheus.client.jetty.JettyStatisticsCollector;
-import java.io.IOException;
 import java.util.ArrayList;
 import java.util.EnumSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Optional;
-import java.util.concurrent.ExecutionException;
 import javax.servlet.DispatcherType;
-import javax.servlet.Filter;
-import javax.servlet.FilterChain;
-import javax.servlet.FilterConfig;
-import javax.servlet.ServletException;
-import javax.servlet.ServletRequest;
-import javax.servlet.ServletResponse;
-import javax.servlet.http.HttpServletResponse;
 import lombok.Getter;
 import org.apache.pulsar.broker.PulsarServerException;
 import org.apache.pulsar.broker.PulsarService;
 import org.apache.pulsar.broker.ServiceConfiguration;
 import org.apache.pulsar.jetty.tls.JettySslContextFactory;
-import org.eclipse.jetty.server.ConnectionFactory;
 import org.eclipse.jetty.server.ConnectionLimit;
-import org.eclipse.jetty.server.ForwardedRequestCustomizer;
 import org.eclipse.jetty.server.Handler;
 import org.eclipse.jetty.server.HttpConfiguration;
 import org.eclipse.jetty.server.HttpConnectionFactory;
-import org.eclipse.jetty.server.ProxyConnectionFactory;
-import org.eclipse.jetty.server.RequestLog;
-import org.eclipse.jetty.server.SecureRequestCustomizer;
 import org.eclipse.jetty.server.Server;
 import org.eclipse.jetty.server.ServerConnector;
-import org.eclipse.jetty.server.SslConnectionFactory;
 import org.eclipse.jetty.server.handler.ContextHandler;
 import org.eclipse.jetty.server.handler.ContextHandlerCollection;
 import org.eclipse.jetty.server.handler.DefaultHandler;
@@ -118,18 +103,9 @@ public class WebService implements AutoCloseable {
 
         Optional<Integer> port = config.getWebServicePort();
         HttpConfiguration httpConfig = new HttpConfiguration();
-        if (config.isWebServiceTrustXForwardedFor()) {
-            httpConfig.addCustomizer(new ForwardedRequestCustomizer());
-        }
         httpConfig.setRequestHeaderSize(pulsar.getConfig().getHttpMaxRequestHeaderSize());
-        HttpConnectionFactory httpConnectionFactory = new HttpConnectionFactory(httpConfig);
         if (port.isPresent()) {
-            List<ConnectionFactory> connectionFactories = new ArrayList<>();
-            if (config.isWebServiceHaProxyProtocolEnabled()) {
-                connectionFactories.add(new ProxyConnectionFactory());
-            }
-            connectionFactories.add(httpConnectionFactory);
-            httpConnector = new ServerConnector(server, connectionFactories.toArray(new ConnectionFactory[0]));
+            httpConnector = new ServerConnector(server, new HttpConnectionFactory(httpConfig));
             httpConnector.setPort(port.get());
             httpConnector.setHost(pulsar.getBindAddress());
             connectors.add(httpConnector);
@@ -168,18 +144,7 @@ public class WebService implements AutoCloseable {
                             config.getWebServiceTlsProtocols(),
                             config.getTlsCertRefreshCheckDurationSec());
                 }
-                List<ConnectionFactory> connectionFactories = new ArrayList<>();
-                if (config.isWebServiceHaProxyProtocolEnabled()) {
-                    connectionFactories.add(new ProxyConnectionFactory());
-                }
-                connectionFactories.add(new SslConnectionFactory(sslCtxFactory, httpConnectionFactory.getProtocol()));
-                connectionFactories.add(httpConnectionFactory);
-                // org.eclipse.jetty.server.AbstractConnectionFactory.getFactories contains similar logic
-                // this is needed for TLS authentication
-                if (httpConfig.getCustomizer(SecureRequestCustomizer.class) == null) {
-                    httpConfig.addCustomizer(new SecureRequestCustomizer());
-                }
-                httpsConnector = new ServerConnector(server, connectionFactories.toArray(new ConnectionFactory[0]));
+                httpsConnector = new ServerConnector(server, sslCtxFactory, new HttpConnectionFactory(httpConfig));
                 httpsConnector.setPort(tlsPort.get());
                 httpsConnector.setHost(pulsar.getBindAddress());
                 connectors.add(httpsConnector);
@@ -237,7 +202,6 @@ public class WebService implements AutoCloseable {
         private final FilterHolder authenticationFilterHolder;
         FilterInitializer(PulsarService pulsarService) {
             ServiceConfiguration config = pulsarService.getConfiguration();
-
             if (config.getMaxConcurrentHttpRequests() > 0) {
                 FilterHolder filterHolder = new FilterHolder(QoSFilter.class);
                 filterHolder.setInitParameter("maxRequests", String.valueOf(config.getMaxConcurrentHttpRequests()));
@@ -249,11 +213,8 @@ public class WebService implements AutoCloseable {
                         new RateLimitingFilter(config.getHttpRequestsMaxPerSecond())));
             }
 
-            // wait until the PulsarService is ready to serve incoming requests
-            filterHolders.add(
-                    new FilterHolder(new WaitUntilPulsarServiceIsReadyForIncomingRequestsFilter(pulsarService)));
-
-            boolean brokerInterceptorEnabled = pulsarService.getBrokerInterceptor() != null;
+            boolean brokerInterceptorEnabled =
+                    pulsarService.getBrokerInterceptor() != null && !config.isDisableBrokerInterceptors();
             if (brokerInterceptorEnabled) {
                 ExceptionHandler handler = new ExceptionHandler();
                 // Enable PreInterceptFilter only when interceptors are enabled
@@ -294,56 +255,19 @@ public class WebService implements AutoCloseable {
             }
         }
 
-        // Filter that waits until the PulsarService is ready to serve incoming requests
-        private static class WaitUntilPulsarServiceIsReadyForIncomingRequestsFilter implements Filter {
-            private final PulsarService pulsarService;
-
-            public WaitUntilPulsarServiceIsReadyForIncomingRequestsFilter(PulsarService pulsarService) {
-                this.pulsarService = pulsarService;
-            }
-
-            @Override
-            public void init(FilterConfig filterConfig) throws ServletException {
-
-            }
-
-            @Override
-            public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain)
-                    throws IOException, ServletException {
-                try {
-                    // Wait until the PulsarService is ready to serve incoming requests
-                    pulsarService.waitUntilReadyForIncomingRequests();
-                } catch (ExecutionException e) {
-                    ((HttpServletResponse) response).sendError(HttpServletResponse.SC_SERVICE_UNAVAILABLE,
-                            "PulsarService failed to start.");
-                    return;
-                } catch (InterruptedException e) {
-                    ((HttpServletResponse) response).sendError(HttpServletResponse.SC_SERVICE_UNAVAILABLE,
-                            "PulsarService is not ready.");
-                    return;
-                }
-                chain.doFilter(request, response);
-            }
-
-            @Override
-            public void destroy() {
-
-            }
-        }
     }
 
     public void addServlet(String path, ServletHolder servletHolder, boolean requiresAuthentication,
                            Map<String, Object> attributeMap) {
-        ServletContextHandler servletContextHandler = new ServletContextHandler(ServletContextHandler.SESSIONS);
+        ServletContextHandler context = new ServletContextHandler(ServletContextHandler.SESSIONS);
         // Notice: each context path should be unique, but there's nothing here to verify that
-        servletContextHandler.setContextPath(path);
-        servletContextHandler.addServlet(servletHolder, MATCH_ALL);
+        context.setContextPath(path);
+        context.addServlet(servletHolder, MATCH_ALL);
         if (attributeMap != null) {
-            attributeMap.forEach(servletContextHandler::setAttribute);
+            attributeMap.forEach(context::setAttribute);
         }
-        filterInitializer.addFilters(servletContextHandler, requiresAuthentication);
-
-        handlers.add(servletContextHandler);
+        filterInitializer.addFilters(context, requiresAuthentication);
+        handlers.add(context);
     }
 
     public void addStaticResources(String basePath, String resourcePath) {
@@ -360,22 +284,15 @@ public class WebService implements AutoCloseable {
     public void start() throws PulsarServerException {
         try {
             RequestLogHandler requestLogHandler = new RequestLogHandler();
-            boolean showDetailedAddresses = pulsar.getConfiguration().getWebServiceLogDetailedAddresses() != null
-                    ? pulsar.getConfiguration().getWebServiceLogDetailedAddresses() :
-                    (pulsar.getConfiguration().isWebServiceHaProxyProtocolEnabled()
-                            || pulsar.getConfiguration().isWebServiceTrustXForwardedFor());
-            RequestLog requestLogger = JettyRequestLogFactory.createRequestLogger(showDetailedAddresses, server);
-            requestLogHandler.setRequestLog(requestLogger);
+            requestLogHandler.setRequestLog(JettyRequestLogFactory.createRequestLogger());
             handlers.add(0, new ContextHandlerCollection());
             handlers.add(requestLogHandler);
 
             ContextHandlerCollection contexts = new ContextHandlerCollection();
             contexts.setHandlers(handlers.toArray(new Handler[handlers.size()]));
 
-            Handler handlerForContexts = GzipHandlerUtil.wrapWithGzipHandler(contexts,
-                    pulsar.getConfig().getHttpServerGzipCompressionExcludedPaths());
             HandlerCollection handlerCollection = new HandlerCollection();
-            handlerCollection.setHandlers(new Handler[] {handlerForContexts, new DefaultHandler(), requestLogHandler});
+            handlerCollection.setHandlers(new Handler[] { contexts, new DefaultHandler(), requestLogHandler });
 
             // Metrics handler
             StatisticsHandler stats = new StatisticsHandler();
@@ -386,6 +303,7 @@ public class WebService implements AutoCloseable {
             } catch (IllegalArgumentException e) {
                 // Already registered. Eg: in unit tests
             }
+            handlers.add(stats);
 
             server.setHandler(stats);
             server.start();
@@ -411,48 +329,28 @@ public class WebService implements AutoCloseable {
 
     @Override
     public void close() throws PulsarServerException {
-        close(true);
-    }
-
-    public void close(boolean waitUtilServerStopped) throws PulsarServerException {
         try {
-            if (waitUtilServerStopped) {
-                doClose();
-            } else {
-                Thread webServiceTerminator = new Thread(() -> {
-                    try {
-                        doClose();
-                    } catch (Exception e) {
-                        log.error("Error while closing web service", e);
-                    }
-                });
-                webServiceTerminator.setName("pulsar-web-service-terminator");
-                webServiceTerminator.start();
+            server.stop();
+            // unregister statistics from Prometheus client's default CollectorRegistry singleton
+            // to prevent memory leaks in tests
+            if (jettyStatisticsCollector != null) {
+                try {
+                    CollectorRegistry.defaultRegistry.unregister(jettyStatisticsCollector);
+                } catch (Exception e) {
+                    // ignore any exception happening in unregister
+                    // exception will be thrown for 2. instance of WebService in tests since
+                    // the register supports a single JettyStatisticsCollector
+                }
+                jettyStatisticsCollector = null;
             }
+            webServiceExecutor.join();
+            this.executorStats.close();
+            log.info("Web service closed");
         } catch (Exception e) {
             throw new PulsarServerException(e);
         }
     }
 
-    private void doClose() throws Exception {
-        server.stop();
-        // unregister statistics from Prometheus client's default CollectorRegistry singleton
-        // to prevent memory leaks in tests
-        if (jettyStatisticsCollector != null) {
-            try {
-                CollectorRegistry.defaultRegistry.unregister(jettyStatisticsCollector);
-            } catch (Exception e) {
-                // ignore any exception happening in unregister
-                // exception will be thrown for 2. instance of WebService in tests since
-                // the register supports a single JettyStatisticsCollector
-            }
-            jettyStatisticsCollector = null;
-        }
-        webServiceExecutor.join();
-        this.executorStats.close();
-        log.info("Web service closed");
-    }
-
     public Optional<Integer> getListenPortHTTP() {
         if (httpConnector != null) {
             return Optional.of(httpConnector.getLocalPort());
