@@ -33,15 +33,18 @@ import java.util.UUID;
 import java.util.concurrent.CompletableFuture;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.ExecutionException;
+import java.util.concurrent.ExecutorService;
 import java.util.concurrent.Executors;
 import java.util.concurrent.ScheduledExecutorService;
 import java.util.concurrent.TimeUnit;
+import lombok.Cleanup;
 import lombok.extern.slf4j.Slf4j;
 import org.apache.commons.lang3.reflect.FieldUtils;
 import org.apache.pulsar.broker.auth.MockedPulsarServiceBaseTest;
 import org.apache.pulsar.broker.service.BrokerServiceException.TopicPoliciesCacheNotInitException;
 import org.apache.pulsar.broker.systopic.SystemTopicClient;
 import org.apache.pulsar.client.admin.PulsarAdminException;
+import org.apache.pulsar.client.api.Schema;
 import org.apache.pulsar.client.impl.Backoff;
 import org.apache.pulsar.client.impl.BackoffBuilder;
 import org.apache.pulsar.common.events.PulsarEvent;
@@ -66,6 +69,8 @@ public class SystemTopicBasedTopicPoliciesServiceTest extends MockedPulsarServic
     private static final String NAMESPACE2 = "system-topic/namespace-2";
     private static final String NAMESPACE3 = "system-topic/namespace-3";
 
+    private static final String NAMESPACE4 = "system-topic/namespace-4";
+
     private static final TopicName TOPIC1 = TopicName.get("persistent", NamespaceName.get(NAMESPACE1), "topic-1");
     private static final TopicName TOPIC2 = TopicName.get("persistent", NamespaceName.get(NAMESPACE1), "topic-2");
     private static final TopicName TOPIC3 = TopicName.get("persistent", NamespaceName.get(NAMESPACE2), "topic-1");
@@ -428,4 +433,35 @@ public class SystemTopicBasedTopicPoliciesServiceTest extends MockedPulsarServic
 
         result.join();
     }
+
+    @Test
+    public void testWriterCache() throws Exception {
+        admin.namespaces().createNamespace(NAMESPACE4);
+        for (int i = 1; i <= 5; i ++) {
+            final String topicName = "persistent://" + NAMESPACE4 + "/testWriterCache" + i;
+            admin.topics().createNonPartitionedTopic(topicName);
+            pulsarClient.newProducer(Schema.STRING).topic(topicName).create().close();
+        }
+        @Cleanup("shutdown")
+        ExecutorService executorService = Executors.newFixedThreadPool(5);
+        for (int i = 1; i <= 5; i ++) {
+            int finalI = i;
+            executorService.execute(() -> {
+                final String topicName = "persistent://" + NAMESPACE4 + "/testWriterCache" + finalI;
+                try {
+                    admin.topicPolicies().setMaxConsumers(topicName, 2);
+                } catch (Exception e) {
+                    throw new RuntimeException(e);
+                }
+            });
+        }
+        SystemTopicBasedTopicPoliciesService service = (SystemTopicBasedTopicPoliciesService) pulsar.getTopicPoliciesService();
+        Assert.assertNotNull(service.getWriterCaches().synchronous().get(NamespaceName.get(NAMESPACE4)));
+        for (int i = 1; i <= 5; i ++) {
+            final String topicName = "persistent://" + NAMESPACE4 + "/testWriterCache" + i;
+            admin.topics().delete(topicName);
+        }
+        admin.namespaces().deleteNamespace(NAMESPACE4);
+        Assert.assertNull(service.getWriterCaches().synchronous().getIfPresent(NamespaceName.get(NAMESPACE4)));
+    }
 }
