@@ -128,8 +128,6 @@ public interface ConsumerBuilder<T> extends Cloneable {
     /**
      * Specify a pattern for topics(not contains the partition suffix) that this consumer subscribes to.
      *
-     * <p>Will ignore the topic domain("persistent://" or "non-persistent://") when pattern matching.
-     *
      * <p>The pattern is applied to subscribe to all topics, within a single namespace, that match the
      * pattern.
      *
@@ -145,9 +143,7 @@ public interface ConsumerBuilder<T> extends Cloneable {
      * Specify a pattern for topics(not contains the partition suffix) that this consumer subscribes to.
      *
      * <p>It accepts a regular expression that is compiled into a pattern internally. E.g.,
-     * "persistent://public/default/pattern-topic-.*" or "public/default/pattern-topic-.*"
-     *
-     * <p>Will ignore the topic domain("persistent://" or "non-persistent://") when pattern matching.
+     * "persistent://public/default/pattern-topic-.*"
      *
      * <p>The pattern is applied to subscribe to all topics, within a single namespace, that match the
      * pattern.
@@ -188,8 +184,6 @@ public interface ConsumerBuilder<T> extends Cloneable {
      * <p>By default, the acknowledgment timeout is disabled (set to `0`, which means infinite).
      * When a consumer with an infinite acknowledgment timeout terminates, any unacknowledged
      * messages that it receives are re-delivered to another consumer.
-     * <p>Since 2.3.0, when a dead letter policy is specified and no ackTimeoutMillis is specified,
-     * the acknowledgment timeout is set to 30 seconds.
      *
      * <p>When enabling acknowledgment timeout, if a message is not acknowledged within the specified timeout,
      * it is re-delivered to the consumer (possibly to a different consumer, in the case of
@@ -204,21 +198,17 @@ public interface ConsumerBuilder<T> extends Cloneable {
     ConsumerBuilder<T> ackTimeout(long ackTimeout, TimeUnit timeUnit);
 
     /**
-     * Enables or disables the acknowledgment receipt feature.
+     * Acknowledgement returns receipt, but the message is not re-sent after getting receipt.
      *
-     * <p>When this feature is enabled, the consumer ensures that acknowledgments are processed by the broker by
-     * waiting for a receipt from the broker. Even when the broker returns a receipt, it doesn't guarantee that the
-     * message won't be redelivered later due to certain implementation details.
-     * It is recommended to use the asynchronous {@link Consumer#acknowledgeAsync(Message)} method for acknowledgment
-     * when this feature is enabled. This is because using the synchronous {@link Consumer#acknowledge(Message)} method
-     * with acknowledgment receipt can cause performance issues due to the round trip to the server, which prevents
-     * pipelining (having multiple messages in-flight). With the asynchronous method, the consumer can continue
-     * consuming other messages while waiting for the acknowledgment receipts.
+     * Configure the acknowledgement timeout mechanism to redeliver the message if it is not acknowledged after
+     * ackTimeout, or to execute a timer task to check the acknowledgement timeout messages during every
+     * ackTimeoutTickTime period.
      *
-     * @param isAckReceiptEnabled {@code true} to enable acknowledgment receipt, {@code false} to disable it
+     * @param isAckReceiptEnabled {@link Boolean} enables acknowledgement for receipt
      * @return the consumer builder instance
      */
     ConsumerBuilder<T> isAckReceiptEnabled(boolean isAckReceiptEnabled);
+
     /**
      * Define the granularity of the ack-timeout redelivery.
      *
@@ -292,21 +282,6 @@ public interface ConsumerBuilder<T> extends Cloneable {
      */
     ConsumerBuilder<T> messageListener(MessageListener<T> messageListener);
 
-    /**
-     * Set the {@link MessageListenerExecutor} to be used for message listeners of <b>current consumer</b>.
-     * <i>(default: use executor from PulsarClient,
-     * {@link org.apache.pulsar.client.impl.PulsarClientImpl#externalExecutorProvider})</i>.
-     *
-     * <p>The listener thread pool is exclusively owned by current consumer
-     * that are using a "listener" model to get messages. For a given internal consumer,
-     * the listener will always be invoked from the same thread, to ensure ordering.
-     *
-     * <p> The caller need to shut down the thread pool after closing the consumer to avoid leaks.
-     * @param messageListenerExecutor the executor of the consumer message listener
-     * @return the consumer builder instance
-     */
-    ConsumerBuilder<T> messageListenerExecutor(MessageListenerExecutor messageListenerExecutor);
-
     /**
      * Sets a {@link CryptoKeyReader}.
      *
@@ -377,9 +352,8 @@ public interface ConsumerBuilder<T> extends Cloneable {
      * size is zero. {@link Consumer#receive()} function call should not be interrupted when the consumer queue size is
      * zero.</li>
      * <li>Doesn't support Batch-Message. If a consumer receives a batch-message, it closes the consumer connection with
-     * the broker and {@link Consumer#receive()} calls will throw {@link PulsarClientException}
-     * while {@link Consumer#receiveAsync()} receives
-     * {@link PulsarClientException} in callback.
+     * the broker and {@link Consumer#receive()} calls remain blocked while {@link Consumer#receiveAsync()} receives
+     * exception in callback.
      *
      * <b> The consumer is not able to receive any further messages unless batch-message in pipeline
      * is removed.</b></li>
@@ -799,33 +773,6 @@ public interface ConsumerBuilder<T> extends Cloneable {
      * If configured with a non-null value, the consumer uses the processor to process the payload, including
      * decoding it to messages and triggering the listener.
      *
-     * <p><b>Special behavior when {@link #receiverQueueSize(int) receiverQueueSize=0}:</b>
-     * When the consumer is configured with {@link #receiverQueueSize(int) receiverQueueSize=0}:
-     * <ul>
-     *   <li>For <b>batch messages</b>:
-     *     <ul>
-     *       <li>The payload processor will <i>not</i> be invoked</li>
-     *       <li>The consumer will <b>immediately close itself</b> upon receiving batch messages</li>
-     *       <li>Pending operations will fail with:
-     *         <ul>
-     *           <li>{@code receive()}: throws {@link PulsarClientException}</li>
-     *           <li>{@code receiveAsync()}: completes the Future with {@link PulsarClientException}</li>
-     *           <li>Message listeners: triggers {@link Consumer#close()} without delivering the message</li>
-     *         </ul>
-     *       </li>
-     *     </ul>
-     *   </li>
-     *   <li>For <b>single messages</b>:
-     *     <ul>
-     *       <li>The payload processor will process messages normally</li>
-     *     </ul>
-     *   </li>
-     * </ul>
-     *
-     *
-     * <p><b>Default behavior {@link #receiverQueueSize(int) receiverQueueSize>0}:</b>
-     * All messages (both single and batched) will be processed by the payload processor.
-     *
      * Default: null
      */
     ConsumerBuilder<T> messagePayloadProcessor(MessagePayloadProcessor payloadProcessor);
@@ -871,16 +818,13 @@ public interface ConsumerBuilder<T> extends Cloneable {
 
     /**
      * If this is enabled, the consumer receiver queue size is initialized as a very small value, 1 by default,
-     * and will double itself until it reaches either the value set by {@link #receiverQueueSize(int)} or the client
-     * memory limit set by {@link ClientBuilder#memoryLimit(long, SizeUnit)}.
-     *
-     * <p>The consumer receiver queue size will double if and only if:
+     * and will double itself until it reaches the value set by {@link #receiverQueueSize(int)}, if and only if:
      * <p>1) User calls receive() and there are no messages in receiver queue.
      * <p>2) The last message we put in the receiver queue took the last space available in receiver queue.
      *
-     * <p>This is disabled by default and currentReceiverQueueSize is initialized as maxReceiverQueueSize.
+     * This is disabled by default and currentReceiverQueueSize is initialized as maxReceiverQueueSize.
      *
-     * <p>The feature should be able to reduce client memory usage.
+     * The feature should be able to reduce client memory usage.
      *
      * @param enabled whether to enable AutoScaledReceiverQueueSize.
      */
