@@ -22,7 +22,6 @@ import java.io.IOException;
 import java.util.Collection;
 import java.util.concurrent.CompletionException;
 import java.util.concurrent.ExecutionException;
-import lombok.Getter;
 import org.apache.pulsar.common.classification.InterfaceAudience;
 import org.apache.pulsar.common.classification.InterfaceStability;
 
@@ -393,17 +392,6 @@ public class PulsarClientException extends IOException {
         }
     }
 
-    /**
-     * Relates to server-side errors:
-     *  ServiceUnitNotReadyException, TopicFencedException and SubscriptionFencedException.
-     */
-    public static class ServiceNotReadyException extends LookupException {
-
-        public ServiceNotReadyException(String msg) {
-            super(msg);
-        }
-    }
-
     /**
      * Connect exception thrown by Pulsar client.
      */
@@ -749,30 +737,6 @@ public class PulsarClientException extends IOException {
         }
     }
 
-    /**
-     * Not supported exception thrown by Pulsar client.
-     */
-    public static class FeatureNotSupportedException extends NotSupportedException {
-
-        @Getter
-        private final FailedFeatureCheck failedFeatureCheck;
-
-        public FeatureNotSupportedException(String msg, FailedFeatureCheck failedFeatureCheck) {
-            super(msg);
-            this.failedFeatureCheck = failedFeatureCheck;
-        }
-    }
-
-    /**
-     * "supports_auth_refresh" was introduced at "2.6" and is no longer supported, so skip this enum.
-     * "supports_broker_entry_metadata" was introduced at "2.8" and is no longer supported, so skip this enum.
-     * "supports_partial_producer" was introduced at "2.10" and is no longer supported, so skip this enum.
-     * "supports_topic_watchers" was introduced at "2.11" and is no longer supported, so skip this enum.
-     */
-    public enum FailedFeatureCheck {
-        SupportsGetPartitionedMetadataWithoutAutoCreation;
-    }
-
     /**
      * Not allowed exception thrown by Pulsar client.
      */
@@ -1151,8 +1115,6 @@ public class PulsarClientException extends IOException {
             newException = new TransactionConflictException(msg);
         } else if (cause instanceof TopicDoesNotExistException) {
             newException = new TopicDoesNotExistException(msg);
-        } else if (cause instanceof SubscriptionNotFoundException) {
-            newException = new SubscriptionNotFoundException(msg);
         } else if (cause instanceof ProducerFencedException) {
             newException = new ProducerFencedException(msg);
         } else if (cause instanceof MemoryBufferIsFullError) {
