@@ -128,9 +128,9 @@ public class AutoClusterFailover implements ServiceUrlProvider {
         try {
             resolver.updateServiceUrl(url);
             InetSocketAddress endpoint = resolver.resolveHost();
-            try (Socket socket = new Socket()) {
-                socket.connect(new InetSocketAddress(endpoint.getHostName(), endpoint.getPort()), TIMEOUT);
-            }
+            Socket socket = new Socket();
+            socket.connect(new InetSocketAddress(endpoint.getHostName(), endpoint.getPort()), TIMEOUT);
+            socket.close();
             return true;
         } catch (Exception e) {
             log.warn("Failed to probe available, url: {}", url, e);
@@ -329,6 +329,7 @@ public class AutoClusterFailover implements ServiceUrlProvider {
 
         @Override
         public AutoClusterFailoverBuilder checkInterval(long interval, TimeUnit timeUnit) {
+            checkArgument(interval >= 0L, "check interval time must not be negative.");
             this.checkIntervalMs = timeUnit.toMillis(interval);
             return this;
         }
