@@ -18,28 +18,6 @@
  */
 package org.apache.pulsar.functions.utils;
 
-import com.google.common.collect.Lists;
-import com.google.gson.Gson;
-import java.util.ArrayList;
-import lombok.Data;
-import lombok.NoArgsConstructor;
-import lombok.experimental.Accessors;
-import org.apache.pulsar.common.functions.ConsumerConfig;
-import org.apache.pulsar.common.functions.FunctionConfig;
-import org.apache.pulsar.common.functions.Resources;
-import org.apache.pulsar.common.io.SinkConfig;
-import org.apache.pulsar.config.validation.ConfigValidationAnnotations;
-import org.apache.pulsar.functions.api.utils.IdentityFunction;
-import org.apache.pulsar.functions.proto.Function;
-import org.testng.annotations.Test;
-
-import java.io.IOException;
-import java.lang.reflect.Field;
-import java.util.Collections;
-import java.util.HashMap;
-import java.util.List;
-import java.util.Map;
-
 import static org.apache.pulsar.common.functions.FunctionConfig.ProcessingGuarantees.ATLEAST_ONCE;
 import static org.apache.pulsar.common.functions.FunctionConfig.ProcessingGuarantees.ATMOST_ONCE;
 import static org.apache.pulsar.common.functions.FunctionConfig.ProcessingGuarantees.EFFECTIVELY_ONCE;
@@ -48,6 +26,29 @@ import static org.testng.Assert.assertFalse;
 import static org.testng.Assert.assertThrows;
 import static org.testng.Assert.assertTrue;
 import static org.testng.Assert.expectThrows;
+import com.google.common.collect.Lists;
+import com.google.gson.Gson;
+import java.io.IOException;
+import java.lang.reflect.Field;
+import java.util.ArrayList;
+import java.util.Collections;
+import java.util.HashMap;
+import java.util.List;
+import java.util.Map;
+import lombok.Data;
+import lombok.NoArgsConstructor;
+import lombok.experimental.Accessors;
+import org.apache.pulsar.common.functions.ConsumerConfig;
+import org.apache.pulsar.common.functions.FunctionConfig;
+import org.apache.pulsar.common.functions.Resources;
+import org.apache.pulsar.common.io.ConnectorDefinition;
+import org.apache.pulsar.common.io.SinkConfig;
+import org.apache.pulsar.config.validation.ConfigValidationAnnotations;
+import org.apache.pulsar.functions.api.Record;
+import org.apache.pulsar.functions.proto.Function;
+import org.apache.pulsar.io.core.Sink;
+import org.apache.pulsar.io.core.SinkContext;
+import org.testng.annotations.Test;
 
 /**
  * Unit test of {@link SinkConfigUtilsTest}.
@@ -62,6 +63,27 @@ public class SinkConfigUtilsTest {
         private String configParameter;
     }
 
+
+    public static class NopSink implements Sink<Object> {
+
+        @Override
+        public void open(Map<String, Object> config, SinkContext sinkContext) throws Exception {
+
+        }
+
+        @Override
+        public void write(Record<Object> record) throws Exception {
+
+        }
+
+        @Override
+        public void close() throws Exception {
+
+        }
+    }
+
+
+
     @Test
     public void testAutoAckConvertFailed() throws IOException {
 
@@ -521,7 +543,7 @@ public class SinkConfigUtilsTest {
         sinkConfig.setNamespace("test-namespace");
         sinkConfig.setName("test-sink");
         sinkConfig.setParallelism(1);
-        sinkConfig.setClassName(IdentityFunction.class.getName());
+        sinkConfig.setClassName(NopSink.class.getName());
         Map<String, ConsumerConfig> inputSpecs = new HashMap<>();
         inputSpecs.put("test-input", ConsumerConfig.builder().isRegexPattern(true).serdeClassName("test-serde").build());
         sinkConfig.setInputSpecs(inputSpecs);
@@ -577,13 +599,16 @@ public class SinkConfigUtilsTest {
         SinkConfig sinkConfig = createSinkConfig();
         sinkConfig.setInputSpecs(null);
         sinkConfig.setTopicsPattern("my-topic-*");
-        SinkConfigUtils.validateAndExtractDetails(sinkConfig, this.getClass().getClassLoader(), null,
+        LoadedFunctionPackage validatableFunction =
+                new LoadedFunctionPackage(this.getClass().getClassLoader(), ConnectorDefinition.class);
+
+        SinkConfigUtils.validateAndExtractDetails(sinkConfig, validatableFunction, null,
                 true);
         sinkConfig.setTimeoutMs(null);
-        SinkConfigUtils.validateAndExtractDetails(sinkConfig, this.getClass().getClassLoader(), null,
+        SinkConfigUtils.validateAndExtractDetails(sinkConfig, validatableFunction, null,
                 true);
         sinkConfig.setTimeoutMs(0L);
-        SinkConfigUtils.validateAndExtractDetails(sinkConfig, this.getClass().getClassLoader(), null,
+        SinkConfigUtils.validateAndExtractDetails(sinkConfig, validatableFunction, null,
                 true);
     }
 }
