@@ -188,12 +188,12 @@ public class KafkaAbstractSinkTest {
                 sink.close();
             }
         };
-        expectThrows(NullPointerException.class, "Kafka topic is not set", openAndClose);
-        config.put("topic", "topic_2");
-        expectThrows(NullPointerException.class, "Kafka bootstrapServers is not set", openAndClose);
+        expectThrows(IllegalArgumentException.class, "bootstrapServers cannot be null", openAndClose);
         config.put("bootstrapServers", "localhost:6667");
-        expectThrows(NullPointerException.class, "Kafka acks mode is not set", openAndClose);
+        expectThrows(IllegalArgumentException.class, "acks cannot be null", openAndClose);
         config.put("acks", "1");
+        expectThrows(IllegalArgumentException.class, "topic cannot be null", openAndClose);
+        config.put("topic", "topic_2");
         config.put("batchSize", "-1");
         expectThrows(IllegalArgumentException.class, "Invalid Kafka Producer batchSize : -1", openAndClose);
         config.put("batchSize", "16384");
